//$Id: oprofile_interface.cpp,v 1.21 2006/05/15 22:09:22 jyeh Exp $

/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2005 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <stdio.h>
#include <iostream>

#include <qfile.h>
#include <qtextstream.h>
#include <qdir.h>
#include <qevent.h>
#include <qprocess.h>
#include <qstringlist.h>
#include <qthread.h>
#include <qcursor.h>
#include <qdatetime.h>
#include <qfileinfo.h>
#include <qmessagebox.h>

#include "application.h"
#include "config.h"
#include "oprofile_interface.h"
//#include "op_config.h"
#include "atuneoptions.h"
#include "symbolengine.h"


namespace OPROFILE_INTERFACE {

	const int MAX_CHAR_PER_LINE = 256;

	QString OP_MOD_NAME = "oprofile";
	QString OP_FS_NAME = "oprofilefs";
	QString OP_MOUNT = "/dev/" + OP_MOD_NAME;

	QString DIR = OP_BASE_DIR;
	QString MOUNT = OP_MOUNT;
	QString LOCK_FILE = DIR + "lock";
	QString CA_OPROFILE_CONTROLLER  = QString(CA_SBIN_DIR) + "/ca_oprofile_controller";
};

using namespace OPROFILE_INTERFACE;
using namespace std;


// Constructor
oprofile_interface::oprofile_interface ()
{
	m_proc=NULL;
	m_pIbsProperties = NULL;
	m_isDaemonStarted = false;
	m_daemon_pid = 0;
	m_numEventCountersAvailable = 0;
	
	CA_OPROFILE_CONTROLLER  = simplifyPath(CA_OPROFILE_CONTROLLER);
}


int oprofile_interface::do_reset ()
{
	QStringList cmd;
	cmd.append("sudo");
	cmd.append(CA_OPROFILE_CONTROLLER);
	cmd.append("--reset");

	return exec_command (cmd);
}


/* Must be called after driver is loaded */
bool oprofile_interface::do_setup ()
{
	QDir dir;
	bool rt = true;

	// Detect number of currently available event counters
	if (QFile::exists(OP_MOUNT + "/" + QString::number(MAX_EVENTNUM_MULTIPLEXING-1)))
		m_numEventCountersAvailable = MAX_EVENTNUM_MULTIPLEXING;
	else
		m_numEventCountersAvailable = 4;  

	/* Clean the OProfile sample directory */
	if(do_reset() != 0)
		rt = false;

	return rt;
}


void oprofile_interface::set_op_events (op_event_property * properties)
{
	if (!properties)
		return;

	for (int i = 0; i < m_numEventCountersAvailable; i++) {
		m_EventProperties[i] = properties[i];
	}
}


void oprofile_interface::set_op_ibs ( op_ibs_property * properties)
{
	m_pIbsProperties = properties;
}


bool oprofile_interface::codeanalyst_service_start()
{
	bool ret = false;

	QStringList cmd;

	cmd.append ("/etc/init.d/codeanalyst");
	cmd.append ("start");

	if(exec_command (cmd) == 0)
		ret = true;
	else
		ret = false;
	
	return ret;
}


bool oprofile_interface::codeanalyst_service_status()
{
	bool ret = false;

	QStringList cmd;

	cmd.append ("/etc/init.d/codeanalyst");
	cmd.append ("status");

	if(exec_command (cmd) == 0)
		ret = true;
	else
		ret = false;
	
	return ret;
}


int oprofile_interface::exec_command (QStringList & args)
{
	if (NULL != m_proc) {
		delete m_proc;
		m_proc = NULL;
	}

	m_proc = new QProcess();
	m_proc->setArguments (args);

#if 0 // DEBUG
	fprintf(stderr,"Command = ");
	QStringList list = m_proc->arguments();
	QStringList::Iterator it = list.begin();
	while( it != list.end() ) {
	fprintf(stderr,"%s ", (*it).data() );
	++it;
	}
	fprintf(stderr,"\n");
#endif	

	int rt = 0;

	if(!m_proc->start ()) {
		rt = -1;
	}

	// Since the process being launch are non-blocking daemon,
	// sysctl, mknod, etc, just wait for the command to finish
	while (rt == 0 && m_proc->isRunning ()) {
	}

	// Check for normal Exit
	if (!m_proc->normalExit ()) {
		rt = -1;
	}

	// Check for exit status
	if (m_proc->exitStatus() != 0) {
		rt = -1;
	}

	m_StdErr = QString(m_proc->readStderr());
	m_StdOut = QString(m_proc->readStdout());

	// Daemon will exit with normal status,
	// event if it failed to start.  Must check
	// stderr to know the correct exit status.
	//if (0 != m_StdErr.length())
	//	rt = -1;

	return rt;
}


void oprofile_interface::generate_op_event_arg (QString & arg)
{
	QString buf = "--events=";
	arg = "";
	for (int i = 0; i < m_numEventCountersAvailable; i++) {
		if (!m_EventProperties[i].op_name.isEmpty ()) {
			buf.sprintf ("%s%s:%u:%i:%llu:%u:%d:%d,",
				buf.data (),
				m_EventProperties[i].op_name.data (),
				(unsigned int)m_EventProperties[i].eventmask,
				i,
				(unsigned long long)m_EventProperties[i].count,
				(unsigned int)m_EventProperties[i].unitmask,
				m_EventProperties[i].usr, 
				m_EventProperties[i].os);
		}
	}
	arg += buf;
}


int oprofile_interface::help_start_daemon (bool novmlinux, 
					   QString & vmlinux_dir,
					   profiling_types active_profiling)
{
	QString event_args;
	QString kernel_range;
	QString buf;
	int rt = OIE_OK;
	m_isDaemonStarted = false; 

	QStringList cmd_OPROFILED_START;
	cmd_OPROFILED_START.append ("sudo");
	cmd_OPROFILED_START.append (CA_OPROFILE_CONTROLLER);
	
	// BEGIN CMD LINE APPENDING.
	QString opd_args;
	
	// Note: this is a string, we don't need the " for arguments.	
	opd_args.append ("--start=");
	opd_args.append ("--session-dir=/var/lib/oprofile ");
	opd_args.append ("--separate-lib=1 ");
	opd_args.append ("--separate-kernel=0 ");
	opd_args.append ("--separate-thread=0 ");
	opd_args.append ("--separate-cpu=1 ");

	//********************************************************
	// VMLINUX stuff	
	buf = "";
	if (!novmlinux) {
		kernel_range = get_kernel_range ();
		buf = "--vmlinux=";
		buf += vmlinux_dir + " ";
		opd_args.append (buf);

		buf = "--kernel-range=";
		buf += kernel_range + " ";
		opd_args.append (buf);
	} else {
		opd_args.append ("--no-vmlinux");
	}
	opd_args.append(" ");

	//********************************************************
	// EVENT stuff
	/* The daemon currently needs "--events" args even when no 
	* event  is selected. */
	buf = "";
	if( !(active_profiling & OP_IBS_FETCH)
	&&  !(active_profiling & OP_IBS_OP))
	{
		generate_op_event_arg (event_args);
		opd_args.append (event_args);
	}

	//********************************************************
#if (OP_VERSION_BASE >= 0x00905)
	// IBS2 stuff
	// --ext-feature=ibs:fetch:ev1,ev2,...,evN:fetch_count:fetch_um|op:ev1,ev2,...,evN:op_count:op_um

	buf = "";
	if( (active_profiling & OP_IBS_FETCH)
	||  (active_profiling & OP_IBS_OP))
		opd_args.append("--ext-feature=ibs:");

	if (active_profiling & OP_IBS_FETCH) {
		buf = QString("fetch:") +
			m_pIbsProperties->fetch_strList.join(",") + ":" +
			QString::number(m_pIbsProperties->fetch_count) + ":" +
			QString::number(m_pIbsProperties->fetch_um);
	} 

	if (active_profiling & OP_IBS_OP) {
		if (active_profiling & OP_IBS_FETCH) 
		buf = buf + QString("\\|");

		buf = buf + QString("op:") +
			m_pIbsProperties->op_strList.join(",") + ":" +
			QString::number(m_pIbsProperties->op_count) + ":" +
			QString::number(m_pIbsProperties->op_um);
	}
	opd_args.append (buf);

#else //(OP_VERSION_BASE >= 0x00905)

	// IBS1 stuff
	buf = "";
	if( (active_profiling & OP_IBS_FETCH)
	||  (active_profiling & OP_IBS_OP))
		opd_args.append("--events= ");

	if (active_profiling & OP_IBS_FETCH) {
		buf.sprintf("--ibs-fetch=%ld ", m_pIbsProperties->fetch_count);
		opd_args.append (buf);
	} 

	if (active_profiling & OP_IBS_OP) {
		buf.sprintf("--ibs-op=%ld ", m_pIbsProperties->op_count);
		opd_args.append (buf);

#if (OPD_NO_DISPATCH_OP_OPTION == 0)
		if(m_pIbsProperties->dispatched_ops)
			opd_args.append ("--ibs-op-dispatch-op ");
#endif
		
#if (OPD_NO_DCMISSINFO_OPTION == 0)
		if(m_pIbsProperties->dcMissInfoEnabled)
			opd_args.append ("--ibs-op-dcmissinfo-enable ");
#endif

	}
#endif //(OP_VERSION_BASE >= 0x00905)

	// END CMD LINE APPENDING.

	cmd_OPROFILED_START.append (opd_args);

#ifdef _DEBUG_
	fprintf(stderr,"DEBUG 0: %s\n",cmd_OPROFILED_START.join(" ").ascii());
#endif
	
	//---------------------------------------------------------
	/* NOTE:
	* Check if daemon is currently running by check 
	* "pidof oprofiled"
	*/
	QStringList cmd_pidof;
	cmd_pidof.append(PIDOF);
	cmd_pidof.append("oprofiled");

	if (exec_command(cmd_pidof) == 0) 
	{
		//		m_StdErr = QString("CodeAnalyst detected a running oprofile daemon ")
		//			+ "(oprofiled)\n.Please make sure oprofile daemon is not "
		//			+ "currently running.\n"; 
		rt = -OIE_DAEMON_RUNNING;
	}

	//---------------------------------------------------------
	/* NOTE:
	* Check if /var/lib/oprofile/lock exists. If not, start daemon. 
	*/
	if (rt == OIE_OK) 
	{
		QFile lockFile (LOCK_FILE);
		if( !lockFile.exists ()) 
		{
			// Starting oprofile daemon 
			if(exec_command (cmd_OPROFILED_START) == 0) 
			{
#if _DEBUG_
				fprintf(stderr,"help_start_daemon: stdout = %s\n",m_StdOut.ascii());
				fprintf(stderr,"help_start_daemon: stderr = %s\n",m_StdErr.ascii());
#endif 
				
				m_isDaemonStarted = true;

				m_StdErr = QString("Failed to check ") + LOCK_FILE 
					+ "\n\nPlease make sure oprofile daemon is not "
					+ "currently running.\n"; 
				rt = -OIE_DAEMON_NOTSTART;

				cmd_OPROFILED_START.clear ();
				qApp->setOverrideCursor( QCursor(Qt::WaitCursor) );

				// Wait for 5 second for the lock file to be created by daemon
				for (int i = 0; i < 10; i++) 
				{
					if (QFile::exists (LOCK_FILE)) 
					{
						m_StdErr = ""; 
						rt = OIE_OK;
						break;
					} else {
						timespec temp;
						temp.tv_sec = 0;
						temp.tv_nsec = FIVE_HUNDRED_MS;
						nanosleep (&temp, NULL);
					}
				}
				qApp->restoreOverrideCursor();

			}else{
#if _DEBUG_
				fprintf(stderr,"help_start_daemon: stdout = %s\n",m_StdOut.ascii());
				fprintf(stderr,"help_start_daemon: stderr = %s\n",m_StdErr.ascii());
				fprintf(stderr,"help_start_daemon: Failed to start daemon\n");
#endif 
				m_StdErr = "Failed to start daemon.\n"; 
				rt = -OIE_DAEMON_NOTSTART;
			}
		}else{
			m_StdErr = QString("File /var/lib/oprofile/lock exists.\n")
				+ "Please remove this file and make sure oprofile "
				+ "daemon is not currently running.\n";
			rt = -OIE_LOCK_EXIST;
		}
	}

	//---------------------------------------------------------
	/* NOTE:
	* If the daemon is created successfully,
	* enable the driver by writting 1 to /dev/oprofile/enable
	* and read back to confirm.
	*/ 
	if (rt == OIE_OK) 
	{
		char wrBuf = '1';
		char rdBuf = '\0';
		QString enable = MOUNT + "/enable";
		QFile enable_file(enable);
		if (enable_file.open (IO_ReadWrite)) {
			// Write 1 to enable
			if (-1 == enable_file.writeBlock (&wrBuf, sizeof (wrBuf))) {
				do_kill_daemon();
				m_StdErr = QString("Oprofile Driver error.\n")
					+ "There was a problem writing to the /dev/oprofile/enable.";
				rt = -OIE_DRV;
			}
			// Read to confirm
			if ( rt == OIE_OK
				&& -1 == enable_file.readBlock (&rdBuf, sizeof (rdBuf))) {
					do_kill_daemon();
					m_StdErr = QString("Oprofile Driver error.\n")
					+ "There was a problem reading from the /dev/oprofile/enable.";
					rt = -OIE_DRV;
			}
			// Compare Read to Write
			if( rt == OIE_OK
				&& rdBuf != wrBuf ) {
					do_kill_daemon();
					m_StdErr = QString("Oprofile Driver error.\n")
					+ "There was a problem enabling driver.";
					rt = -OIE_DRV;
			}

			enable_file.close ();
		}else{
			do_kill_daemon();
			m_StdErr = QString("Oprofile Driver error.\n")
				+ "There was a problem openning /dev/oprofile/enable.";
			rt = -OIE_DRV;
		}
	}

	if (rt == OIE_OK) {
		m_isDaemonStarted = true;
		get_daemon_pid(m_daemon_pid);
	} else {
		m_isDaemonStarted = false;
		m_daemon_pid = 0;
	}

	return rt;
}


bool oprofile_interface::get_daemon_pid(unsigned int &pid) 
{
	QFile lock (LOCK_FILE);
	QString line;
	if (lock.exists() && lock.open(IO_ReadOnly)) {
		lock.readLine(line,MAX_CHAR_PER_LINE);
		lock.close();
		pid = line.toUInt();
		return true;
	}
	return false;		
}


QString & oprofile_interface::get_std_err() 
{
	return m_StdErr;
}


int oprofile_interface::do_start_daemon (bool novmlinux, 
		QString & vmlinux_dir,
		unsigned int buffer_size,
		unsigned int watershed_size, 
		unsigned int cpu_buf_size,
		profiling_types active_profiling,
		unsigned int multiplex_interval)
{
	m_StdErr = "";

	if(!do_sysctl_setup(buffer_size, 
			watershed_size, 
			cpu_buf_size, 
			active_profiling,
			multiplex_interval))
		return -OIE_ERR;

	return help_start_daemon(novmlinux, vmlinux_dir, active_profiling);
}


bool oprofile_interface::set_ctr_param (int event, QString field, unsigned long long int value)
{
	bool rt = true;
	QStringList args;
	QString buf;
	QString st_event;
	QString st_value;


	st_event.setNum (event);
	st_value.setNum (value);

	QString fname;
	fname.sprintf ("%s/%s/%s", MOUNT.data (), st_event.data (), field.data ());
	QFile param_file (fname);;

	if (param_file.open (IO_WriteOnly)) 
	{
		if (-1 == param_file.writeBlock (st_value.data (), st_value.length ()))
		{
			m_StdErr = QString("Cannot write to ") + fname + ".\n";
			rt = false;
		}
		param_file.close ();
	} else {
		m_StdErr = QString("Cannot open ") + fname + ".\n";
		rt = false;
	}
	return rt;
}

bool oprofile_interface::set_param (QString field, int value)
{
	bool rt = true;
	QStringList args;
	QString buf;
	QString st_value;

	st_value.setNum (value);

	QString fname;
	fname.sprintf ("%s/%s", MOUNT.data (), field.data ());
	QFile param_file (fname);;

	if (param_file.open (IO_WriteOnly)) 
	{
		if (-1 == param_file.writeBlock (st_value.data (), st_value.length ()))
		{
			m_StdErr = QString("Cannot write to ") + fname + ".\n";
			rt = false;
		}
		param_file.close ();

	}else {
		m_StdErr = QString("Cannot open ") + fname + ".\n";
		rt = false;
	}	
	return rt;
}


bool oprofile_interface::do_ebs_setup(profiling_types active_profiling)
{
	bool rt = false;

	if (!(active_profiling & OP_EBP)) 
		return true;

	for (int i = 0; i < m_numEventCountersAvailable; i++) {
		if (!m_EventProperties[i].op_name.isEmpty ()) {
			if(!set_ctr_param (i, QString ("enabled"), 1)) goto errOut;
			if(!set_ctr_param (i, QString ("event"), m_EventProperties[i].eventmask)) goto errOut;
			if(!set_ctr_param (i, QString ("count"), m_EventProperties[i].count)) goto errOut;
			if(!set_ctr_param (i, QString ("kernel"), m_EventProperties[i].os)) goto errOut;
			if(!set_ctr_param (i, QString ("user"), m_EventProperties[i].usr)) goto errOut;
			if(!set_ctr_param (i, QString ("unit_mask"), m_EventProperties[i].unitmask)) goto errOut;
		}
	}
	rt = true;
errOut:
	return rt;
}


bool oprofile_interface::do_ibs_setup(profiling_types active_profiling)
{
	// Setup IBS Stuff if exist
	QFile ibs_fetch_dir("/dev/oprofile/ibs_fetch");
	QFile ibs_op_dir("/dev/oprofile/ibs_op");
	bool rt = false;

	if(ibs_fetch_dir.exists() && ibs_op_dir.exists()) {
		// Reset IBS parameters
		if(!set_param ("ibs_fetch/enable", 0)) goto errOut;
		if(!set_param ("ibs_op/enable" , 0)) goto errOut;

		// Setup IBS parameters
		if (active_profiling & OP_IBS_FETCH) {
			if(!set_param ("ibs_fetch/max_count", m_pIbsProperties->fetch_count)) goto errOut;
			if(!set_param ("ibs_fetch/rand_enable", 1)) goto errOut;
			if(!set_param ("ibs_fetch/enable", 1)) goto errOut;
		} 
		if (active_profiling & OP_IBS_OP) {
			if(!set_param ("ibs_op/max_count", m_pIbsProperties->op_count)) goto errOut;
			if(!set_param ("ibs_op/enable", 1)) goto errOut;
			// Check if dispatched_ops is available
			struct stat tmp;
			if(0 == stat("/dev/oprofile/ibs_op/dispatched_ops",&tmp))
				if(!set_param ("ibs_op/dispatched_ops", m_pIbsProperties->dispatched_ops)) goto errOut;
		}
	}
	rt = true;
errOut:
	return rt;
}


bool oprofile_interface::do_sysctl_setup (unsigned int buffer_size,
				unsigned int watershed_size, 
				unsigned int cpu_buf_size, 
				profiling_types active_profiling,
				unsigned int multiplex_interval)
{
	QFile mux_ctrl(QString("/dev/oprofile/")+ MUX_DRIVER_INTERFACE);
	QStringList args;
	QString buf;
	bool rt = true;
	
	/* Setting /dev/oprofile/buffer_size */
	if(buffer_size != 0 && !set_param("buffer_size", buffer_size)) {
		m_StdErr = QString("Cannot configure /dev/oprofile/buffer_size.\n");
		goto errOut;
	}

	/* Setting /dev/oprofile/buffer_watershed */
	if(buffer_size != 0 && !set_param("buffer_watershed", watershed_size)) {
		m_StdErr = QString("Cannot configure /dev/oprofile/buffer_watershed.\n");
		goto errOut;
	}

	/* Setting /dev/oprofile/cpu_buffer_size */
	if(cpu_buf_size != 0 && !set_param ("cpu_buffer_size", cpu_buf_size)) {
		m_StdErr = QString("Cannot configure /dev/oprofile/cpu_buffer_size.\n");
		goto errOut;
	}

	/* Setting up multiplexing we need:
 	 * 	- MUX interval > 0 
 	 * 	- m_numEventCountersAvailable == MAX_EVENTNUM_MULTIPLEXING
 	 * 	- /dev/oprofile/time_slice present	
 	 */
	if (multiplex_interval != 0) {
		if (!mux_ctrl.exists() || m_numEventCountersAvailable == MAX_EVENTNUM ) {
			/* This usually means driver mismatch */
			
			// Force number of events to just MAX_EVENTNUM
			m_numEventCountersAvailable = MAX_EVENTNUM;
	
			// Try to clear it if exist.
			if(mux_ctrl.exists()) set_param (MUX_DRIVER_INTERFACE, 0);
		} else {
			if(!set_param (MUX_DRIVER_INTERFACE, multiplex_interval)) {
				/* This usually means driver error */
				m_StdErr = QString("Cannot configure /dev/oprofile/")
					+ MUX_DRIVER_INTERFACE +".\n";
				goto errOut;
			}
		}
	} else {
		// Try to clear it if exist.
		if(mux_ctrl.exists()) set_param (MUX_DRIVER_INTERFACE, multiplex_interval);
	}

	/* Initialize all counters */
	for (int i = 0; i < m_numEventCountersAvailable; i++) {
		m_StdErr = QString("Cannot clear /dev/oprofile/0,1,2,3...\n");
		if(!set_ctr_param (i, QString ("enabled"), 0)) goto errOut;
		if(!set_ctr_param (i, QString ("event"), 0)) goto errOut;
		if(!set_ctr_param (i, QString ("count"), 0)) goto errOut;
	}

	if (!do_ebs_setup(active_profiling)) {
		m_StdErr = QString("Failed to setup driver for EBP.\n");
		goto errOut;
	}

	if (!do_ibs_setup(active_profiling)) {
		m_StdErr = QString("Failed to setup driver for IBS.\n");
		goto errOut;
	}

	m_StdErr = "";
	return rt;

errOut:	
	m_StdErr += QString("\nPlease also make sure that :\n")
		+ "    1. /dev/oprofile/... exists.\n"
		+ "    2. User is in the \"amdca\" group.\n\n"
		+ "Otherwise, please do one of the followings:\n"
		+ "    1. Run \"/etc/init.d/codeanalyst start\" as root.\n"
		+ "    2. Or reboot the system.\n";
	rt = false;
	return rt;

}


QString oprofile_interface::get_kernel_range ()
{
	char buf[LONG_STR_MAX * 2 + 1];
	char buf2[LONG_STR_MAX];
	char buf3[LONG_STR_MAX];
	CATuneOptions ao;
	QString vmlinux_dir = "";
	QString range = "";
	ObjDump objdump;
	SymbolEngine symengine;
	string line;
	bfd_size_type size = 0;
	bfd_vma str_addr;

	ao.getVmlinuxDir (vmlinux_dir);

	int symengineCode = symengine.open(vmlinux_dir.data ());

	if ((SymbolEngine::OKAY == symengineCode) ||
			(SymbolEngine::NO_SYMBOLS == symengineCode) ||
			SymbolEngine::NO_SYMBOL_TABLE == symengineCode) {

		symengine.getSecBase (&str_addr, ".text");
		symengine.getSecSize (&size, ".text"); 
		bfd_vma end_addr = str_addr + size;
		snprintf(buf2, (LONG_STR_MAX-1), LONG_FORMAT, (long unsigned int)str_addr);
		snprintf(buf3, (LONG_STR_MAX-1), LONG_FORMAT, (long unsigned int)end_addr);
		snprintf(buf, (LONG_STR_MAX * 2), "%s,%s", buf2, buf3);
	}

	range = buf;
	symengine.closeEngine();
	return range;
}


/* NOTE:
* Here, we write 1 to /dev/oprofile/dump, and then we MUST check
* if the dump is completed by checking if daemon write 1 to
* /var/lib/oprofile/complete_dump
*/
void oprofile_interface::do_dump ()
{
	char buf = '1';
	QString dump = MOUNT + "/dump";
	QFile dump_file (dump);

	QString complete_dump = "/var/lib/oprofile/complete_dump";
	QFileInfo complete_dump_info (complete_dump);
	QDateTime last_dump;
	QDateTime cur_dump;

	timespec temp;
	temp.tv_sec = 0;
	temp.tv_nsec = FIVE_HUNDRED_MS;

	//---------------------------
	// Get previous complete_dump time
	if(complete_dump_info.exists())
		last_dump = complete_dump_info.lastModified();
	else
		last_dump = QDateTime::currentDateTime();

	//---------------------------
	// Write 1 to /dev/oprofile/dump
	if ((! dump_file.open (IO_WriteOnly)) 
		|| (-1 == dump_file.writeBlock (&buf, sizeof (buf))))
		QMessageBox::critical(NULL, "Writing error", "There was a problem "
		"writing a block to the dump");

	dump_file.close ();

	//---------------------------
	// Wait for complete_dump to be refresh.

	// Max wait time = 3 sec	
	int i = 6; 
	while(1)
	{
		complete_dump_info.refresh();
		cur_dump = complete_dump_info.lastModified();
		if (cur_dump.isValid() && (cur_dump > last_dump))
			break;
		else
			if(i != 0) {	
				nanosleep (&temp, NULL);
				i--;
			} else {
				break;	
			}		
	}
}



void oprofile_interface::do_stop ()
{
	QString command;

	command = "echo \"0\">" + MOUNT + "/enable";
	system(command.data());
}

void oprofile_interface::do_resume_daemon()
{
	/* NOTE:
	* To Resume, write 1 to /dev/oprofile/enable file
	*/
	char buf = '1';
	QString enable = MOUNT + "/enable";
	QFile enable_file(enable);
	if (enable_file.open (IO_WriteOnly)) {
		if (-1 == enable_file.writeBlock (&buf, sizeof (buf))) {
			QMessageBox::information (NULL, "Writing error", "There was a "
				"problem writing a block to the "
				"enable");
		}
		enable_file.close ();
	}
}


/* Pause the daemon */
void oprofile_interface::do_pause_daemon ()
{
	/* NOTE:
	* To Resume, write 0 to /dev/oprofile/enable file
	*/
	char buf = '0';
	QString enable = MOUNT + "/enable";
	QFile enable_file(enable);
	if (enable_file.open (IO_WriteOnly)) 
	{
		if (-1 == enable_file.writeBlock (&buf, sizeof (buf))) 
		{
			QMessageBox::information (NULL, "Writing error", "There was a "
				"problem writing a block to the "
				"enable");
		}
		enable_file.close ();
	}
}

bool oprofile_interface::remove_lock_file()
{
	bool rt = false;

	QStringList cmd_OPROFILE_RMLOCK;
	cmd_OPROFILE_RMLOCK.append("sudo");
	cmd_OPROFILE_RMLOCK.append(CA_OPROFILE_CONTROLLER);
	cmd_OPROFILE_RMLOCK.append("--rmlock");
	if (exec_command(cmd_OPROFILE_RMLOCK) == 0)	
	{
		rt = true;
	}
	return rt;
}

/* Note: The logic in this function should be similar to opcontrol script*/
void oprofile_interface::do_kill_daemon ()
{
	QFile lock (LOCK_FILE);
	QStringList cmd_pidof;
	cmd_pidof.append(PIDOF);
	cmd_pidof.append("oprofiled");
	bool isDaemonRunning = false;
	QString daemonPid;

	// Sanity check if daemon is running.
	if (exec_command(cmd_pidof) == 0)
	{
		isDaemonRunning = true;
		daemonPid = m_StdOut.simplifyWhiteSpace();
	}

	/* NOTE:
	* We have to handle these scenario
	* 1.1 No Daemon, No Lock, No Daemon Started
	* 1.2 No Daemon, No Lock, Daemon Started
	* 2.  No Daemon, Have Lock
	* 3.  Have Daemon, Have different lock, Daemon Started
	* 4.  Have Daemon, No Lock
	* 5.  Have Daemon, Have Lock
	*/ 

	// 1.2 No Daemon, No Lock, No Daemon Started
	if (!isDaemonRunning && !lock.exists() && !m_isDaemonStarted)
		return;

	// 1.2 No Daemon, No Lock, Daemon Started
	if (!isDaemonRunning && !lock.exists() && m_isDaemonStarted)
	{
		QMessageBox::warning(NULL, "Oprofile Daemon Warning",
			QString("Warning:\n") +
			"Oprofile daemon died unexpectedly.\n" +
			"Profile result might be inaccurate.\n" +
			"Please rerun the profile");
		m_isDaemonStarted = false;
		return; 
	}

	// 2. No Daemon, Have Lock 
	if (!isDaemonRunning && lock.exists()) {
		QMessageBox::warning(NULL, "Oprofile Daemon Warning",
			QString("Warning:\n") +
			"Oprofile daemon died unexpectedly.\n" +
			"Profile result might be inaccurate.\n" +
			"Please rerun the profile");

		remove_lock_file();
		m_isDaemonStarted = false;
		return;
	}

	// 3 Have Daemon, Have different lock, Daemon Started 
	unsigned int cur_daemon_pid;
	if (m_isDaemonStarted
	&&  get_daemon_pid(cur_daemon_pid)
	&&  cur_daemon_pid != m_daemon_pid) {
		QMessageBox::warning(NULL, "Oprofile Daemon Warning",
			QString("Warning:\n") +
			"A different Oprofile daemon is running.\n" +
			"Profile result might be inaccurate.\n" +
			"Please rerun the profile");
		return;
	}
	
	// 4. Have Daemon, No Lock. This is a bad case, we can't kill it.
	if (!lock.exists()) {
		QMessageBox::warning(NULL, "Oprofile Daemon Warning",
			"Warning:\nLock file does not exist but Oprofile daemon is running.");
	}

	// START KILLING DAEMON

	m_isDaemonStarted = false;

	// First, dump samples
	if (isDaemonRunning)
		do_dump(); 

	// Second, stop daemon and driver
	if (isDaemonRunning)
		do_pause_daemon(); 

	QStringList cmd_OPROFILED_TERM;
	cmd_OPROFILED_TERM.append("sudo");
	cmd_OPROFILED_TERM.append(CA_OPROFILE_CONTROLLER);
	cmd_OPROFILED_TERM.append("--term");

	QStringList cmd_OPROFILED_KILL;
	cmd_OPROFILED_KILL.append("sudo");
	cmd_OPROFILED_KILL.append(CA_OPROFILE_CONTROLLER);
	cmd_OPROFILED_KILL.append("--kill");

	if(exec_command (cmd_OPROFILED_TERM) == -1)
	{
		QMessageBox::warning(NULL, "Oprofile Daemon Warning",
			"Warning:\nca_oprofile_controller return error. "
			"Oprofile daemon might have exitted unexpectedly, "
			"please rerun the profile.");
		return;
	}

	/* NOTE: 
	* Give oprofile daemon 20 seconds to exit before killing.
	* IBS generates much more data and 20 seems to be appropriate.
	* The number may need adustment */

	int i = 20;
	timespec temp;

	temp.tv_sec = 1;
	temp.tv_nsec = 0;
	while (exec_command(cmd_pidof) != -1) {
		if (0 == i) {
			exec_command(cmd_OPROFILED_KILL);
			break;
		}
		exec_command(cmd_OPROFILED_TERM);
		//		fprintf(stderr,"DEBUG: Stopping Daemon count down %d\n",i);
		nanosleep (&temp, NULL);
		i--;
	}

	/* NOTE:
	* At this point there should be no daemon running, we might still have
	* stale lock.  MUST REMOVE IT.
	*/
	if(lock.exists())
	{
		//		fprintf(stderr,"DEBUG: Stale lock file.\n");
		if(!remove_lock_file())
			QMessageBox::warning(NULL, "CodeAnalyst Warning",
			"Warning:\n"
			"Cannot remove /var/lib/oprofile/lock");
	}
}


void oprofile_interface::customEvent (QCustomEvent * pCustEvent)
{
	Q_UNUSED (pCustEvent);
}

QString oprofile_interface::simplifyPath(QString path)
{
	QString ret = path;
	while(ret.contains("//"))
		ret = ret.replace("//","/");
	return ret;	
}

bool oprofile_interface::havePermissionToProfile()
{
	bool ret = false;
	int count;

	// NOTE: We need to parse output of sudo -S -l 
	// for 3 different cases:
	//
	// Case root User:
	// User root may run the following commands on this host:
	//	(ALL) ALL
	//
	// Case Good User:
	// User suravee may run the following commands on this host:
	//	(root) NOPASSWD: /usr/sbin/ca_oprofile_controller
	//
	// Case Bad User:
	// [sudo] password for suravee1: 

	QProcess proc;
	QStringList cmd;
	QString line_stdout;

	cmd.append ("sudo");
	cmd.append ("-S");
	cmd.append ("-l");
	proc.setArguments (cmd);

	if(!proc.start ()) {
		return ret;
	}

	// Wait for the 1 sec for the stdout
	count = 2;
	timespec temp;
	temp.tv_sec = 0;
	temp.tv_nsec = FIVE_HUNDRED_MS;
	while(proc.isRunning ()) {
		count--;
		nanosleep (&temp, NULL);
		if(proc.canReadLineStdout()) 
			break;	
		if(count == 0) {
			// Write to stdin to stop 
			// process gracefully.
			proc.writeToStdin("");	
			return ret;
		}
	}
		
	// Read first line of stdout	
	line_stdout = QString(proc.readLineStdout());

	// Handling bad case	
	if(!line_stdout.startsWith("User ")) {
		proc.kill();
		return ret;
	}

	// Processing stdout
	while( (line_stdout = proc.readLineStdout()) 
			!= QString::null ) 
	{
		// For root user
		if(line_stdout.contains("(ALL) ALL")) {
			proc.kill();
			ret = true;
			break;
		}

		// For non-root user
		QString tmp = simplifyPath(line_stdout.section(" ",-1));

		if(tmp == CA_OPROFILE_CONTROLLER) {
			proc.kill();
			ret = true;
			break;
		} 
	}

	return ret;
}


bool oprofile_interface::checkIbsSupportInDaemon()
{
	bool ret = false;
	QString command;
	
#if (OP_VERSION_BASE == 0x00903)
	command = QString(OP_BINDIR) +
			"/opcontrol --help 2>&1 " +
			"| grep ibs-fetch 2> /dev/null > /dev/null";
	
	ret = (system(command.ascii()) == 0)? true: false;
#endif

#if (OP_VERSION_BASE >= 0x00905)
	
	command = QString(OP_BINDIR) +
			"/oprofiled --help 2>&1 " +
			"| grep ext-feature 2> /dev/null > /dev/null";
	ret = (system(command.ascii()) == 0)? true: false;
#endif
	return ret;
}

bool oprofile_interface::checkMuxSupportInDaemon()
{
	// NOTE: This check allows only the CodeAnalyst Oprofile
	//       to use MUX.
	QString command = QString(OP_BINDIR) +
			"opcontrol --help 2>&1 " +
			"| grep multiplexing-time 2> /dev/null > /dev/null";
	return ((system(command.ascii()) == 0)? true: false);
}


bool oprofile_interface::checkMuxSupportInDriver()
{
	QFile mux_ctrl(QString("/dev/oprofile/")+ MUX_DRIVER_INTERFACE);
	return ((mux_ctrl.exists())? true: false);
}
